@@ -9,15 +9,15 @@ import numpy as np
 from numpy import ma
 
 import matplotlib as mpl
-from matplotlib import _api, docstring
+from matplotlib import _api, _docstring
 from matplotlib.backend_bases import MouseButton
+from matplotlib.text import Text
 import matplotlib.path as mpath
 import matplotlib.ticker as ticker
 import matplotlib.cm as cm
 import matplotlib.colors as mcolors
 import matplotlib.collections as mcoll
 import matplotlib.font_manager as font_manager
-import matplotlib.text as text
 import matplotlib.cbook as cbook
 import matplotlib.patches as mpatches
 import matplotlib.transforms as mtransforms
@@ -31,7 +31,8 @@ import matplotlib.transforms as mtransforms
 # per level.
 
 
-class ClabelText(text.Text):
+@_api.deprecated("3.7", alternative="Text.set_transform_rotates_text")
+class ClabelText(Text):
     """
     Unlike the ordinary text, the get_rotation returns an updated
     angle in the pixel coordinate assuming that the input rotation is
@@ -150,10 +151,8 @@ class ContourLabeler:
             or minus 90 degrees from level.
 
         use_clabeltext : bool, default: False
-            If ``True``, `.ClabelText` class (instead of `.Text`) is used to
-            create labels. `ClabelText` recalculates rotation angles
-            of texts during the drawing time, therefore this can be used if
-            aspect of the axes changes.
+            If ``True``, use `.Text.set_transform_rotates_text` to ensure that
+            label rotation is updated whenever the axes aspect changes.
 
         zorder : float or None, default: ``(2 + contour.get_zorder())``
             zorder of the contour labels.
@@ -202,10 +201,7 @@ class ContourLabeler:
         self.labelLevelList = levels
         self.labelIndiceList = indices
 
-        self.labelFontProps = font_manager.FontProperties()
-        self.labelFontProps.set_size(fontsize)
-        font_size_pts = self.labelFontProps.get_size_in_points()
-        self.labelFontSizeList = [font_size_pts] * len(levels)
+        self._label_font_props = font_manager.FontProperties(size=fontsize)
 
         if colors is None:
             self.labelMappable = self
@@ -218,10 +214,10 @@ class ContourLabeler:
 
         self.labelXYs = []
 
-        if np.iterable(self.labelManual):
-            for x, y in self.labelManual:
+        if np.iterable(manual):
+            for x, y in manual:
                 self.add_label_near(x, y, inline, inline_spacing)
-        elif self.labelManual:
+        elif manual:
             print('Select label locations manually using first mouse button.')
             print('End manual selection with second mouse button.')
             if not inline:
@@ -234,8 +230,23 @@ class ContourLabeler:
         else:
             self.labels(inline, inline_spacing)
 
-        self.labelTextsList = cbook.silent_list('text.Text', self.labelTexts)
-        return self.labelTextsList
+        return cbook.silent_list('text.Text', self.labelTexts)
+
+    @_api.deprecated("3.7", alternative="cs.labelTexts[0].get_font()")
+    @property
+    def labelFontProps(self):
+        return self._label_font_props
+
+    @_api.deprecated("3.7", alternative=(
+        "[cs.labelTexts[0].get_font().get_size()] * len(cs.labelLevelList)"))
+    @property
+    def labelFontSizeList(self):
+        return [self._label_font_props.get_size()] * len(self.labelLevelList)
+
+    @_api.deprecated("3.7", alternative="cs.labelTexts")
+    @property
+    def labelTextsList(self):
+        return cbook.silent_list('text.Text', self.labelTexts)
 
     def print_label(self, linecontour, labelwidth):
         """Return whether a contour is long enough to hold a label."""
@@ -248,59 +259,21 @@ class ContourLabeler:
         return any((x - loc[0]) ** 2 + (y - loc[1]) ** 2 < thresh
                    for loc in self.labelXYs)
 
-    @_api.deprecated("3.4")
-    def get_label_coords(self, distances, XX, YY, ysize, lw):
-        """
-        Return x, y, and the index of a label location.
-
-        Labels are plotted at a location with the smallest
-        deviation of the contour from a straight line
-        unless there is another label nearby, in which case
-        the next best place on the contour is picked up.
-        If all such candidates are rejected, the beginning
-        of the contour is chosen.
-        """
-        hysize = int(ysize / 2)
-        adist = np.argsort(distances)
-
-        for ind in adist:
-            x, y = XX[ind][hysize], YY[ind][hysize]
-            if self.too_close(x, y, lw):
-                continue
-            return x, y, ind
-
-        ind = adist[0]
-        x, y = XX[ind][hysize], YY[ind][hysize]
-        return x, y, ind
-
     def _get_nth_label_width(self, nth):
         """Return the width of the *nth* label, in pixels."""
         fig = self.axes.figure
-        return (
-            text.Text(0, 0,
-                      self.get_text(self.labelLevelList[nth], self.labelFmt),
-                      figure=fig,
-                      size=self.labelFontSizeList[nth],
-                      fontproperties=self.labelFontProps)
-            .get_window_extent(mpl.tight_layout.get_renderer(fig)).width)
-
-    @_api.deprecated("3.5")
-    def get_label_width(self, lev, fmt, fsize):
-        """Return the width of the label in points."""
-        if not isinstance(lev, str):
-            lev = self.get_text(lev, fmt)
-        fig = self.axes.figure
-        width = (text.Text(0, 0, lev, figure=fig,
-                           size=fsize, fontproperties=self.labelFontProps)
-                 .get_window_extent(mpl.tight_layout.get_renderer(fig)).width)
-        width *= 72 / fig.dpi
-        return width
+        renderer = fig._get_renderer()
+        return (Text(0, 0,
+                     self.get_text(self.labelLevelList[nth], self.labelFmt),
+                     figure=fig, fontproperties=self._label_font_props)
+                .get_window_extent(renderer).width)
 
+    @_api.deprecated("3.7", alternative="Artist.set")
     def set_label_props(self, label, text, color):
         """Set the label properties - color, fontsize, text."""
         label.set_text(text)
         label.set_color(color)
-        label.set_fontproperties(self.labelFontProps)
+        label.set_fontproperties(self._label_font_props)
         label.set_clip_box(self.axes.bbox)
 
     def get_text(self, lev, fmt):
@@ -440,57 +413,32 @@ class ContourLabeler:
 
         return rotation, nlc
 
-    def _get_label_text(self, x, y, rotation):
-        dx, dy = self.axes.transData.inverted().transform((x, y))
-        t = text.Text(dx, dy, rotation=rotation,
-                      horizontalalignment='center',
-                      verticalalignment='center', zorder=self._clabel_zorder)
-        return t
-
-    def _get_label_clabeltext(self, x, y, rotation):
-        # x, y, rotation is given in pixel coordinate. Convert them to
-        # the data coordinate and create a label using ClabelText
-        # class. This way, the rotation of the clabel is along the
-        # contour line always.
-        transDataInv = self.axes.transData.inverted()
-        dx, dy = transDataInv.transform((x, y))
-        drotation = transDataInv.transform_angles(np.array([rotation]),
-                                                  np.array([[x, y]]))
-        t = ClabelText(dx, dy, rotation=drotation[0],
-                       horizontalalignment='center',
-                       verticalalignment='center', zorder=self._clabel_zorder)
-
-        return t
-
-    def _add_label(self, t, x, y, lev, cvalue):
-        color = self.labelMappable.to_rgba(cvalue, alpha=self.alpha)
-
-        _text = self.get_text(lev, self.labelFmt)
-        self.set_label_props(t, _text, color)
+    def add_label(self, x, y, rotation, lev, cvalue):
+        """Add contour label without `.Text.set_transform_rotates_text`."""
+        data_x, data_y = self.axes.transData.inverted().transform((x, y))
+        t = Text(
+            data_x, data_y,
+            text=self.get_text(lev, self.labelFmt),
+            rotation=rotation,
+            horizontalalignment='center', verticalalignment='center',
+            zorder=self._clabel_zorder,
+            color=self.labelMappable.to_rgba(cvalue, alpha=self.alpha),
+            fontproperties=self._label_font_props,
+            clip_box=self.axes.bbox)
         self.labelTexts.append(t)
         self.labelCValues.append(cvalue)
         self.labelXYs.append((x, y))
-
         # Add label to plot here - useful for manual mode label selection
         self.axes.add_artist(t)
 
-    def add_label(self, x, y, rotation, lev, cvalue):
-        """
-        Add contour label using :class:`~matplotlib.text.Text` class.
-        """
-        t = self._get_label_text(x, y, rotation)
-        self._add_label(t, x, y, lev, cvalue)
-
     def add_label_clabeltext(self, x, y, rotation, lev, cvalue):
-        """
-        Add contour label using :class:`ClabelText` class.
-        """
-        # x, y, rotation is given in pixel coordinate. Convert them to
-        # the data coordinate and create a label using ClabelText
-        # class. This way, the rotation of the clabel is along the
-        # contour line always.
-        t = self._get_label_clabeltext(x, y, rotation)
-        self._add_label(t, x, y, lev, cvalue)
+        """Add contour label with `.Text.set_transform_rotates_text`."""
+        self.add_label(x, y, rotation, lev, cvalue)
+        # Grab the last added text, and reconfigure its rotation.
+        t = self.labelTexts[-1]
+        data_rotation, = self.axes.transData.inverted().transform_angles(
+            [rotation], [[x, y]])
+        t.set(rotation=data_rotation, transform_rotates_text=True)
 
     def add_label_near(self, x, y, inline=True, inline_spacing=5,
                        transform=None):
@@ -554,9 +502,7 @@ class ContourLabeler:
             paths.pop(segmin)
 
             # Add paths if not empty or single point
-            for n in nlc:
-                if len(n) > 1:
-                    paths.append(mpath.Path(n))
+            paths.extend([mpath.Path(n) for n in nlc if len(n) > 1])
 
     def pop_label(self, index=-1):
         """Defaults to removing last label, but any index can be supplied"""
@@ -610,6 +556,10 @@ class ContourLabeler:
             if inline:
                 paths[:] = additions
 
+    def remove(self):
+        for text in self.labelTexts:
+            text.remove()
+
 
 def _is_closed_polygon(X):
     """
@@ -637,7 +587,7 @@ def _find_closest_point_on_path(xys, p):
         Projection of *p* onto *xys*.
     imin : (int, int)
         Consecutive indices of vertices of segment in *xys* where *proj* is.
-        Segments are considered as including their end-points; i.e if the
+        Segments are considered as including their end-points; i.e. if the
         closest point on the path is a node in *xys* with index *i*, this
         returns ``(i-1, i)``.  For the special case where *xys* is a single
         point, this returns ``(0, 0)``.
@@ -656,7 +606,7 @@ def _find_closest_point_on_path(xys, p):
     return (d2s[imin], projs[imin], (imin, imin+1))
 
 
-docstring.interpd.update(contour_set_attributes=r"""
+_docstring.interpd.update(contour_set_attributes=r"""
 Attributes
 ----------
 ax : `~matplotlib.axes.Axes`
@@ -675,7 +625,7 @@ layers : array
 """)
 
 
-@docstring.dedent_interpd
+@_docstring.dedent_interpd
 class ContourSet(cm.ScalarMappable, ContourLabeler):
     """
     Store a set of contour lines or filled regions.
@@ -724,7 +674,7 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
                  hatches=(None,), alpha=None, origin=None, extent=None,
                  cmap=None, colors=None, norm=None, vmin=None, vmax=None,
                  extend='neither', antialiased=None, nchunk=0, locator=None,
-                 transform=None,
+                 transform=None, negative_linestyles=None,
                  **kwargs):
         """
         Draw contour lines or filled regions, depending on
@@ -809,6 +759,13 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
 
         self._transform = transform
 
+        self.negative_linestyles = negative_linestyles
+        # If negative_linestyles was not defined as a keyword argument, define
+        # negative_linestyles with rcParams
+        if self.negative_linestyles is None:
+            self.negative_linestyles = \
+                mpl.rcParams['contour.negative_linestyle']
+
         kwargs = self._process_args(*args, **kwargs)
         self._process_levels()
 
@@ -928,10 +885,7 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
             )
 
     def get_transform(self):
-        """
-        Return the :class:`~matplotlib.transforms.Transform`
-        instance used by this ContourSet.
-        """
+        """Return the `.Transform` instance used by this ContourSet."""
         if self._transform is None:
             self._transform = self.axes.transData
         elif (not isinstance(self._transform, mtransforms.Transform)
@@ -1078,13 +1032,13 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
     def _make_paths(self, segs, kinds):
         """
         Create and return Path objects for the specified segments and optional
-        kind codes.  segs is a list of numpy arrays, each array is either a
+        kind codes.  *segs* is a list of numpy arrays, each array is either a
         closed line loop or open line strip of 2D points with a shape of
-        (npoints, 2).  kinds is either None or a list (with the same length as
-        segs) of numpy arrays, each array is of shape (npoints,) and contains
-        the kinds codes for the corresponding line in segs.  If kinds is None
-        then the Path constructor creates the kind codes assuming that the line
-        is an open strip.
+        (npoints, 2).  *kinds* is either None or a list (with the same length
+        as *segs*) of numpy arrays, each array is of shape (npoints,) and
+        contains the kind codes for the corresponding line in *segs*.  If
+        *kinds* is None then the Path constructor creates the kind codes
+        assuming that the line is an open strip.
         """
         if kinds is None:
             return [mpath.Path(seg) for seg in segs]
@@ -1163,33 +1117,28 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
 
         return lev[i0:i1]
 
-    def _process_contour_level_args(self, args):
+    def _process_contour_level_args(self, args, z_dtype):
         """
         Determine the contour levels and store in self.levels.
         """
         if self.levels is None:
-            if len(args) == 0:
-                levels_arg = 7  # Default, hard-wired.
-            else:
+            if args:
                 levels_arg = args[0]
+            elif np.issubdtype(z_dtype, bool):
+                if self.filled:
+                    levels_arg = [0, .5, 1]
+                else:
+                    levels_arg = [.5]
+            else:
+                levels_arg = 7  # Default, hard-wired.
         else:
             levels_arg = self.levels
         if isinstance(levels_arg, Integral):
             self.levels = self._autolev(levels_arg)
         else:
-            self.levels = np.asarray(levels_arg).astype(np.float64)
-
-        if not self.filled:
-            inside = (self.levels > self.zmin) & (self.levels < self.zmax)
-            levels_in = self.levels[inside]
-            if len(levels_in) == 0:
-                self.levels = [self.zmin]
-                _api.warn_external(
-                    "No contour levels were found within the data range.")
-
+            self.levels = np.asarray(levels_arg, np.float64)
         if self.filled and len(self.levels) < 2:
             raise ValueError("Filled contours require at least 2 levels.")
-
         if len(self.levels) > 1 and np.min(np.diff(self.levels)) <= 0.0:
             raise ValueError("Contour levels must be increasing")
 
@@ -1302,11 +1251,10 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
         if linestyles is None:
             tlinestyles = ['solid'] * Nlev
             if self.monochrome:
-                neg_ls = mpl.rcParams['contour.negative_linestyle']
                 eps = - (self.zmax - self.zmin) * 1e-15
                 for i, lev in enumerate(self.levels):
                     if lev < eps:
-                        tlinestyles[i] = neg_ls
+                        tlinestyles[i] = self.negative_linestyles
         else:
             if isinstance(linestyles, str):
                 tlinestyles = [linestyles] * Nlev
@@ -1384,6 +1332,7 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
         d2min = np.inf
         conmin = None
         segmin = None
+        imin = None
         xmin = None
         ymin = None
 
@@ -1411,8 +1360,13 @@ class ContourSet(cm.ScalarMappable, ContourLabeler):
 
         return (conmin, segmin, imin, xmin, ymin, d2min)
 
+    def remove(self):
+        super().remove()
+        for coll in self.collections:
+            coll.remove()
 
-@docstring.dedent_interpd
+
+@_docstring.dedent_interpd
 class QuadContourSet(ContourSet):
     """
     Create and store a set of contour lines or filled regions.
@@ -1423,7 +1377,7 @@ class QuadContourSet(ContourSet):
     %(contour_set_attributes)s
     """
 
-    def _process_args(self, *args, corner_mask=None, **kwargs):
+    def _process_args(self, *args, corner_mask=None, algorithm=None, **kwargs):
         """
         Process args and kwargs.
         """
@@ -1436,21 +1390,31 @@ class QuadContourSet(ContourSet):
             contour_generator = args[0]._contour_generator
             self._mins = args[0]._mins
             self._maxs = args[0]._maxs
+            self._algorithm = args[0]._algorithm
         else:
-            import matplotlib._contour as _contour
+            import contourpy
+
+            if algorithm is None:
+                algorithm = mpl.rcParams['contour.algorithm']
+            mpl.rcParams.validate["contour.algorithm"](algorithm)
+            self._algorithm = algorithm
 
             if corner_mask is None:
-                corner_mask = mpl.rcParams['contour.corner_mask']
+                if self._algorithm == "mpl2005":
+                    # mpl2005 does not support corner_mask=True so if not
+                    # specifically requested then disable it.
+                    corner_mask = False
+                else:
+                    corner_mask = mpl.rcParams['contour.corner_mask']
             self._corner_mask = corner_mask
 
             x, y, z = self._contour_args(args, kwargs)
 
-            _mask = ma.getmask(z)
-            if _mask is ma.nomask or not _mask.any():
-                _mask = None
-
-            contour_generator = _contour.QuadContourGenerator(
-                x, y, z.filled(), _mask, self._corner_mask, self.nchunk)
+            contour_generator = contourpy.contour_generator(
+                x, y, z, name=self._algorithm, corner_mask=self._corner_mask,
+                line_type=contourpy.LineType.SeparateCode,
+                fill_type=contourpy.FillType.OuterCode,
+                chunk_size=self.nchunk)
 
             t = self.get_transform()
 
@@ -1476,17 +1440,16 @@ class QuadContourSet(ContourSet):
             fn = 'contourf'
         else:
             fn = 'contour'
-        Nargs = len(args)
-        if Nargs <= 2:
-            z = ma.asarray(args[0], dtype=np.float64)
+        nargs = len(args)
+        if nargs <= 2:
+            z, *args = args
+            z = ma.asarray(z)
             x, y = self._initialize_x_y(z)
-            args = args[1:]
-        elif Nargs <= 4:
-            x, y, z = self._check_xyz(args[:3], kwargs)
-            args = args[3:]
+        elif nargs <= 4:
+            x, y, z_orig, *args = args
+            x, y, z = self._check_xyz(x, y, z_orig, kwargs)
         else:
-            raise TypeError("Too many arguments to %s; see help(%s)" %
-                            (fn, fn))
+            raise _api.nargs_error(fn, takes="from 1 to 4", given=nargs)
         z = ma.masked_invalid(z, copy=False)
         self.zmax = float(z.max())
         self.zmin = float(z.min())
@@ -1494,20 +1457,19 @@ class QuadContourSet(ContourSet):
             z = ma.masked_where(z <= 0, z)
             _api.warn_external('Log scale: values of z <= 0 have been masked')
             self.zmin = float(z.min())
-        self._process_contour_level_args(args)
+        self._process_contour_level_args(args, z.dtype)
         return (x, y, z)
 
-    def _check_xyz(self, args, kwargs):
+    def _check_xyz(self, x, y, z, kwargs):
         """
         Check that the shapes of the input arrays match; if x and y are 1D,
         convert them to 2D using meshgrid.
         """
-        x, y = args[:2]
         x, y = self.axes._process_unit_info([("x", x), ("y", y)], kwargs)
 
         x = np.asarray(x, dtype=np.float64)
         y = np.asarray(y, dtype=np.float64)
-        z = ma.asarray(args[2], dtype=np.float64)
+        z = ma.asarray(z)
 
         if z.ndim != 2:
             raise TypeError(f"Input z must be 2D, not {z.ndim}D")
@@ -1583,7 +1545,7 @@ class QuadContourSet(ContourSet):
         return np.meshgrid(x, y)
 
 
-docstring.interpd.update(contour_doc="""
+_docstring.interpd.update(contour_doc="""
 `.contour` and `.contourf` draw contour lines and filled contours,
 respectively.  Except as noted, function signatures and return values
 are the same for both versions.
@@ -1604,14 +1566,15 @@ X, Y : array-like, optional
     ``X = range(N)``, ``Y = range(M)``.
 
 Z : (M, N) array-like
-    The height values over which the contour is drawn.
+    The height values over which the contour is drawn.  Color-mapping is
+    controlled by *cmap*, *norm*, *vmin*, and *vmax*.
 
 levels : int or array-like, optional
     Determines the number and positions of the contour lines / regions.
 
     If an int *n*, use `~matplotlib.ticker.MaxNLocator`, which tries
     to automatically choose no more than *n+1* "nice" contour levels
-    between *vmin* and *vmax*.
+    between minimum and maximum numeric values of *Z*.
 
     If array-like, draw contour lines at the specified levels.
     The values must be in increasing order.
@@ -1647,21 +1610,20 @@ colors : color string or sequence of colors, optional
 alpha : float, default: 1
     The alpha blending value, between 0 (transparent) and 1 (opaque).
 
-cmap : str or `.Colormap`, default: :rc:`image.cmap`
-    A `.Colormap` instance or registered colormap name. The colormap
-    maps the level values to colors.
+%(cmap_doc)s
 
-    If both *colors* and *cmap* are given, an error is raised.
+    This parameter is ignored if *colors* is set.
 
-norm : `~matplotlib.colors.Normalize`, optional
-    If a colormap is used, the `.Normalize` instance scales the level
-    values to the canonical colormap range [0, 1] for mapping to
-    colors. If not given, the default linear scaling is used.
+%(norm_doc)s
 
-vmin, vmax : float, optional
-    If not *None*, either or both of these values will be supplied to
-    the `.Normalize` instance, overriding the default color scaling
-    based on *levels*.
+    This parameter is ignored if *colors* is set.
+
+%(vmin_vmax_doc)s
+
+    If *vmin* or *vmax* are not given, the default color scaling is based on
+    *levels*.
+
+    This parameter is ignored if *colors* is set.
 
 origin : {*None*, 'upper', 'lower', 'image'}, default: None
     Determines the orientation and exact position of *Z* by specifying
@@ -1758,14 +1720,27 @@ linewidths : float or array-like, default: :rc:`contour.linewidth`
 linestyles : {*None*, 'solid', 'dashed', 'dashdot', 'dotted'}, optional
     *Only applies to* `.contour`.
 
-    If *linestyles* is *None*, the default is 'solid' unless the lines
-    are monochrome.  In that case, negative contours will take their
-    linestyle from :rc:`contour.negative_linestyle` setting.
+    If *linestyles* is *None*, the default is 'solid' unless the lines are
+    monochrome. In that case, negative contours will instead take their
+    linestyle from the *negative_linestyles* argument.
 
-    *linestyles* can also be an iterable of the above strings
-    specifying a set of linestyles to be used. If this
-    iterable is shorter than the number of contour levels
-    it will be repeated as necessary.
+    *linestyles* can also be an iterable of the above strings specifying a set
+    of linestyles to be used. If this iterable is shorter than the number of
+    contour levels it will be repeated as necessary.
+
+negative_linestyles : {*None*, 'solid', 'dashed', 'dashdot', 'dotted'}, \
+                       optional
+    *Only applies to* `.contour`.
+
+    If *linestyles* is *None* and the lines are monochrome, this argument
+    specifies the line style for negative contours.
+
+    If *negative_linestyles* is *None*, the default is taken from
+    :rc:`contour.negative_linestyles`.
+
+    *negative_linestyles* can also be an iterable of the above strings
+    specifying a set of linestyles to be used. If this iterable is shorter than
+    the number of contour levels it will be repeated as necessary.
 
 hatches : list[str], optional
     *Only applies to* `.contourf`.
@@ -1775,6 +1750,15 @@ hatches : list[str], optional
     Hatching is supported in the PostScript, PDF, SVG and Agg
     backends only.
 
+algorithm : {'mpl2005', 'mpl2014', 'serial', 'threaded'}, optional
+    Which contouring algorithm to use to calculate the contour lines and
+    polygons. The algorithms are implemented in
+    `ContourPy <https://github.com/contourpy/contourpy>`_, consult the
+    `ContourPy documentation <https://contourpy.readthedocs.io>`_ for
+    further information.
+
+    The default is taken from :rc:`contour.algorithm`.
+
 data : indexable object, optional
     DATA_PARAMETER_PLACEHOLDER
 
@@ -1795,5 +1779,5 @@ Notes
 3. `.contour` and `.contourf` use a `marching squares
    <https://en.wikipedia.org/wiki/Marching_squares>`_ algorithm to
    compute contour locations.  More information can be found in
-   the source ``src/_contour.h``.
-""")
+   `ContourPy documentation <https://contourpy.readthedocs.io>`_.
+""" % _docstring.interpd.params)
