@@ -1,23 +1,25 @@
 r"""
 :mod:`~matplotlib.gridspec` contains classes that help to layout multiple
-`~axes.Axes` in a grid-like pattern within a figure.
+`~.axes.Axes` in a grid-like pattern within a figure.
 
 The `GridSpec` specifies the overall grid structure. Individual cells within
 the grid are referenced by `SubplotSpec`\s.
 
-See the tutorial :ref:`sphx_glr_tutorials_intermediate_gridspec.py` for a
-comprehensive usage guide.
+Often, users need not access this module directly, and can use higher-level
+methods like `~.pyplot.subplots`, `~.pyplot.subplot_mosaic` and
+`~.Figure.subfigures`. See the tutorial
+:doc:`/tutorials/intermediate/arranging_axes` for a guide.
 """
 
 import copy
 import logging
+from numbers import Integral
 
 import numpy as np
 
 import matplotlib as mpl
-from matplotlib import _pylab_helpers, cbook, tight_layout, rcParams
+from matplotlib import _api, _pylab_helpers, _tight_layout
 from matplotlib.transforms import Bbox
-import matplotlib._layoutbox as layoutbox
 
 _log = logging.getLogger(__name__)
 
@@ -39,19 +41,25 @@ class GridSpecBase:
             relative width of ``width_ratios[i] / sum(width_ratios)``.
             If not given, all columns will have the same width.
         height_ratios : array-like of length *nrows*, optional
-            Defines the relative heights of the rows. Each column gets a
+            Defines the relative heights of the rows. Each row gets a
             relative height of ``height_ratios[i] / sum(height_ratios)``.
             If not given, all rows will have the same height.
         """
+        if not isinstance(nrows, Integral) or nrows <= 0:
+            raise ValueError(
+                f"Number of rows must be a positive integer, not {nrows!r}")
+        if not isinstance(ncols, Integral) or ncols <= 0:
+            raise ValueError(
+                f"Number of columns must be a positive integer, not {ncols!r}")
         self._nrows, self._ncols = nrows, ncols
         self.set_height_ratios(height_ratios)
         self.set_width_ratios(width_ratios)
 
     def __repr__(self):
         height_arg = (', height_ratios=%r' % (self._row_height_ratios,)
-                      if self._row_height_ratios is not None else '')
+                      if len(set(self._row_height_ratios)) != 1 else '')
         width_arg = (', width_ratios=%r' % (self._col_width_ratios,)
-                     if self._col_width_ratios is not None else '')
+                     if len(set(self._col_width_ratios)) != 1 else '')
         return '{clsname}({nrows}, {ncols}{optionals})'.format(
             clsname=self.__class__.__name__,
             nrows=self._nrows,
@@ -97,7 +105,9 @@ class GridSpecBase:
         *width_ratios* must be of length *ncols*. Each column gets a relative
         width of ``width_ratios[i] / sum(width_ratios)``.
         """
-        if width_ratios is not None and len(width_ratios) != self._ncols:
+        if width_ratios is None:
+            width_ratios = [1] * self._ncols
+        elif len(width_ratios) != self._ncols:
             raise ValueError('Expected the given number of width ratios to '
                              'match the number of columns of the grid')
         self._col_width_ratios = width_ratios
@@ -117,7 +127,9 @@ class GridSpecBase:
         *height_ratios* must be of length *nrows*. Each row gets a relative
         height of ``height_ratios[i] / sum(height_ratios)``.
         """
-        if height_ratios is not None and len(height_ratios) != self._nrows:
+        if height_ratios is None:
+            height_ratios = [1] * self._nrows
+        elif len(height_ratios) != self._nrows:
             raise ValueError('Expected the given number of height ratios to '
                              'match the number of rows of the grid')
         self._row_height_ratios = height_ratios
@@ -130,6 +142,7 @@ class GridSpecBase:
         """
         return self._row_height_ratios
 
+    @_api.delete_parameter("3.7", "raw")
     def get_grid_positions(self, fig, raw=False):
         """
         Return the positions of the grid cells in figure coordinates.
@@ -174,22 +187,16 @@ class GridSpecBase:
         # calculate accumulated heights of columns
         cell_h = tot_height / (nrows + hspace*(nrows-1))
         sep_h = hspace * cell_h
-        if self._row_height_ratios is not None:
-            norm = cell_h * nrows / sum(self._row_height_ratios)
-            cell_heights = [r * norm for r in self._row_height_ratios]
-        else:
-            cell_heights = [cell_h] * nrows
+        norm = cell_h * nrows / sum(self._row_height_ratios)
+        cell_heights = [r * norm for r in self._row_height_ratios]
         sep_heights = [0] + ([sep_h] * (nrows-1))
         cell_hs = np.cumsum(np.column_stack([sep_heights, cell_heights]).flat)
 
         # calculate accumulated widths of rows
         cell_w = tot_width / (ncols + wspace*(ncols-1))
         sep_w = wspace * cell_w
-        if self._col_width_ratios is not None:
-            norm = cell_w * ncols / sum(self._col_width_ratios)
-            cell_widths = [r * norm for r in self._col_width_ratios]
-        else:
-            cell_widths = [cell_w] * ncols
+        norm = cell_w * ncols / sum(self._col_width_ratios)
+        cell_widths = [r * norm for r in self._col_width_ratios]
         sep_widths = [0] + ([sep_w] * (ncols-1))
         cell_ws = np.cumsum(np.column_stack([sep_widths, cell_widths]).flat)
 
@@ -197,6 +204,27 @@ class GridSpecBase:
         fig_lefts, fig_rights = (left + cell_ws).reshape((-1, 2)).T
         return fig_bottoms, fig_tops, fig_lefts, fig_rights
 
+    @staticmethod
+    def _check_gridspec_exists(figure, nrows, ncols):
+        """
+        Check if the figure already has a gridspec with these dimensions,
+        or create a new one
+        """
+        for ax in figure.get_axes():
+            gs = ax.get_gridspec()
+            if gs is not None:
+                if hasattr(gs, 'get_topmost_subplotspec'):
+                    # This is needed for colorbar gridspec layouts.
+                    # This is probably OK because this whole logic tree
+                    # is for when the user is doing simple things with the
+                    # add_subplot command.  For complicated layouts
+                    # like subgridspecs the proper gridspec is passed in...
+                    gs = gs.get_topmost_subplotspec().get_gridspec()
+                if gs.get_geometry() == (nrows, ncols):
+                    return gs
+        # else gridspec not found:
+        return GridSpec(nrows, ncols, figure=figure)
+
     def __getitem__(self, key):
         """Create and return a `.SubplotSpec` instance."""
         nrows, ncols = self.get_geometry()
@@ -224,8 +252,8 @@ class GridSpecBase:
         if isinstance(key, tuple):
             try:
                 k1, k2 = key
-            except ValueError:
-                raise ValueError("unrecognized subplot spec")
+            except ValueError as err:
+                raise ValueError("Unrecognized subplot spec") from err
             num1, num2 = np.ravel_multi_index(
                 [_normalize(k1, nrows, 0), _normalize(k2, ncols, 1)],
                 (nrows, ncols))
@@ -234,6 +262,59 @@ class GridSpecBase:
 
         return SubplotSpec(self, num1, num2)
 
+    def subplots(self, *, sharex=False, sharey=False, squeeze=True,
+                 subplot_kw=None):
+        """
+        Add all subplots specified by this `GridSpec` to its parent figure.
+
+        See `.Figure.subplots` for detailed documentation.
+        """
+
+        figure = self.figure
+
+        if figure is None:
+            raise ValueError("GridSpec.subplots() only works for GridSpecs "
+                             "created with a parent figure")
+
+        if not isinstance(sharex, str):
+            sharex = "all" if sharex else "none"
+        if not isinstance(sharey, str):
+            sharey = "all" if sharey else "none"
+
+        _api.check_in_list(["all", "row", "col", "none", False, True],
+                           sharex=sharex, sharey=sharey)
+        if subplot_kw is None:
+            subplot_kw = {}
+        # don't mutate kwargs passed by user...
+        subplot_kw = subplot_kw.copy()
+
+        # Create array to hold all axes.
+        axarr = np.empty((self._nrows, self._ncols), dtype=object)
+        for row in range(self._nrows):
+            for col in range(self._ncols):
+                shared_with = {"none": None, "all": axarr[0, 0],
+                               "row": axarr[row, 0], "col": axarr[0, col]}
+                subplot_kw["sharex"] = shared_with[sharex]
+                subplot_kw["sharey"] = shared_with[sharey]
+                axarr[row, col] = figure.add_subplot(
+                    self[row, col], **subplot_kw)
+
+        # turn off redundant tick labeling
+        if sharex in ["col", "all"]:
+            for ax in axarr.flat:
+                ax._label_outer_xaxis(check_patch=True)
+        if sharey in ["row", "all"]:
+            for ax in axarr.flat:
+                ax._label_outer_yaxis(check_patch=True)
+
+        if squeeze:
+            # Discarding unneeded dimensions that equal 1.  If we only have one
+            # subplot, just return it instead of a 1-element array.
+            return axarr.item() if axarr.size == 1 else axarr.squeeze()
+        else:
+            # Returned axis array will be always 2-d, even if nrows=ncols=1.
+            return axarr
+
 
 class GridSpec(GridSpecBase):
     """
@@ -242,6 +323,8 @@ class GridSpec(GridSpecBase):
     The location of the grid cells is determined in a similar way to
     `~.figure.SubplotParams` using *left*, *right*, *top*, *bottom*, *wspace*
     and *hspace*.
+
+    Indexing a GridSpec instance returns a `.SubplotSpec`.
     """
     def __init__(self, nrows, ncols, figure=None,
                  left=None, bottom=None, right=None, top=None,
@@ -253,8 +336,8 @@ class GridSpec(GridSpecBase):
         nrows, ncols : int
             The number of rows and columns of the grid.
 
-        figure : `~.figure.Figure`, optional
-            Only used for constrained layout to create a proper layoutbox.
+        figure : `.Figure`, optional
+            Only used for constrained layout to create a proper layoutgrid.
 
         left, right, top, bottom : float, optional
             Extent of the subplots as a fraction of figure width or height.
@@ -280,7 +363,7 @@ class GridSpec(GridSpecBase):
             If not given, all columns will have the same width.
 
         height_ratios : array-like of length *nrows*, optional
-            Defines the relative heights of the rows. Each column gets a
+            Defines the relative heights of the rows. Each row gets a
             relative height of ``height_ratios[i] / sum(height_ratios)``.
             If not given, all rows will have the same height.
 
@@ -293,37 +376,12 @@ class GridSpec(GridSpecBase):
         self.hspace = hspace
         self.figure = figure
 
-        GridSpecBase.__init__(self, nrows, ncols,
-                              width_ratios=width_ratios,
-                              height_ratios=height_ratios)
-
-        if self.figure is None or not self.figure.get_constrained_layout():
-            self._layoutbox = None
-        else:
-            self.figure.init_layoutbox()
-            self._layoutbox = layoutbox.LayoutBox(
-                parent=self.figure._layoutbox,
-                name='gridspec' + layoutbox.seq_id(),
-                artist=self)
-        # by default the layoutbox for a gridspec will fill a figure.
-        # but this can change below if the gridspec is created from a
-        # subplotspec. (GridSpecFromSubplotSpec)
+        super().__init__(nrows, ncols,
+                         width_ratios=width_ratios,
+                         height_ratios=height_ratios)
 
     _AllowedKeys = ["left", "bottom", "right", "top", "wspace", "hspace"]
 
-    def __getstate__(self):
-        state = self.__dict__
-        try:
-            state.pop('_layoutbox')
-        except KeyError:
-            pass
-        return state
-
-    def __setstate__(self, state):
-        self.__dict__ = state
-        # layoutboxes don't survive pickling...
-        self._layoutbox = None
-
     def update(self, **kwargs):
         """
         Update the subplot parameters of the grid.
@@ -346,26 +404,15 @@ class GridSpec(GridSpecBase):
                 raise AttributeError(f"{k} is an unknown keyword")
         for figmanager in _pylab_helpers.Gcf.figs.values():
             for ax in figmanager.canvas.figure.axes:
-                # copied from Figure.subplots_adjust
-                if not isinstance(ax, mpl.axes.SubplotBase):
-                    # Check if sharing a subplots axis
-                    if isinstance(ax._sharex, mpl.axes.SubplotBase):
-                        if ax._sharex.get_subplotspec().get_gridspec() == self:
-                            ax._sharex.update_params()
-                            ax._set_position(ax._sharex.figbox)
-                    elif isinstance(ax._sharey, mpl.axes.SubplotBase):
-                        if ax._sharey.get_subplotspec().get_gridspec() == self:
-                            ax._sharey.update_params()
-                            ax._set_position(ax._sharey.figbox)
-                else:
+                if ax.get_subplotspec() is not None:
                     ss = ax.get_subplotspec().get_topmost_subplotspec()
                     if ss.get_gridspec() == self:
-                        ax.update_params()
-                        ax._set_position(ax.figbox)
+                        ax._set_position(
+                            ax.get_subplotspec().get_position(ax.figure))
 
     def get_subplot_params(self, figure=None):
         """
-        Return the `~.SubplotParams` for the GridSpec.
+        Return the `.SubplotParams` for the GridSpec.
 
         In order of precedence the values are taken from
 
@@ -374,7 +421,8 @@ class GridSpec(GridSpecBase):
         - :rc:`figure.subplot.*`
         """
         if figure is None:
-            kw = {k: rcParams["figure.subplot."+k] for k in self._AllowedKeys}
+            kw = {k: mpl.rcParams["figure.subplot."+k]
+                  for k in self._AllowedKeys}
             subplotpars = mpl.figure.SubplotParams(**kw)
         else:
             subplotpars = copy.copy(figure.subplotpars)
@@ -399,31 +447,27 @@ class GridSpec(GridSpecBase):
 
         Parameters
         ----------
+        figure : `.Figure`
+            The figure.
+        renderer :  `.RendererBase` subclass, optional
+            The renderer to be used.
         pad : float
             Padding between the figure edge and the edges of subplots, as a
             fraction of the font-size.
         h_pad, w_pad : float, optional
             Padding (height/width) between edges of adjacent subplots.
             Defaults to *pad*.
-        rect : tuple of 4 floats, optional
+        rect : tuple (left, bottom, right, top), default: None
             (left, bottom, right, top) rectangle in normalized figure
             coordinates that the whole subplots area (including labels) will
-            fit into.  Default is (0, 0, 1, 1).
+            fit into. Default (None) is the whole figure.
         """
-
-        subplotspec_list = tight_layout.get_subplotspec_list(
-            figure.axes, grid_spec=self)
-        if None in subplotspec_list:
-            cbook._warn_external("This figure includes Axes that are not "
-                                 "compatible with tight_layout, so results "
-                                 "might be incorrect.")
-
         if renderer is None:
-            renderer = tight_layout.get_renderer(figure)
-
-        kwargs = tight_layout.get_tight_layout_figure(
-            figure, figure.axes, subplotspec_list, renderer,
-            pad=pad, h_pad=h_pad, w_pad=w_pad, rect=rect)
+            renderer = figure._get_renderer()
+        kwargs = _tight_layout.get_tight_layout_figure(
+            figure, figure.axes,
+            _tight_layout.get_subplotspec_list(figure.axes, grid_spec=self),
+            renderer, pad=pad, h_pad=h_pad, w_pad=w_pad, rect=rect)
         if kwargs:
             self.update(**kwargs)
 
@@ -438,38 +482,36 @@ class GridSpecFromSubplotSpec(GridSpecBase):
                  wspace=None, hspace=None,
                  height_ratios=None, width_ratios=None):
         """
-        The number of rows and number of columns of the grid need to
-        be set. An instance of SubplotSpec is also needed to be set
-        from which the layout parameters will be inherited. The wspace
-        and hspace of the layout can be optionally specified or the
-        default values (from the figure or rcParams) will be used.
+        Parameters
+        ----------
+        nrows, ncols : int
+            Number of rows and number of columns of the grid.
+        subplot_spec : SubplotSpec
+            Spec from which the layout parameters are inherited.
+        wspace, hspace : float, optional
+            See `GridSpec` for more details. If not specified default values
+            (from the figure or rcParams) are used.
+        height_ratios : array-like of length *nrows*, optional
+            See `GridSpecBase` for details.
+        width_ratios : array-like of length *ncols*, optional
+            See `GridSpecBase` for details.
         """
         self._wspace = wspace
         self._hspace = hspace
         self._subplot_spec = subplot_spec
-        GridSpecBase.__init__(self, nrows, ncols,
-                              width_ratios=width_ratios,
-                              height_ratios=height_ratios)
-        # do the layoutboxes
-        subspeclb = subplot_spec._layoutbox
-        if subspeclb is None:
-            self._layoutbox = None
-        else:
-            # OK, this is needed to divide the figure.
-            self._layoutbox = subspeclb.layout_from_subplotspec(
-                    subplot_spec,
-                    name=subspeclb.name + '.gridspec' + layoutbox.seq_id(),
-                    artist=self)
+        self.figure = self._subplot_spec.get_gridspec().figure
+        super().__init__(nrows, ncols,
+                         width_ratios=width_ratios,
+                         height_ratios=height_ratios)
 
     def get_subplot_params(self, figure=None):
-        """Return a dictionary of subplot layout parameters.
-        """
+        """Return a dictionary of subplot layout parameters."""
         hspace = (self._hspace if self._hspace is not None
                   else figure.subplotpars.hspace if figure is not None
-                  else rcParams["figure.subplot.hspace"])
+                  else mpl.rcParams["figure.subplot.hspace"])
         wspace = (self._wspace if self._wspace is not None
                   else figure.subplotpars.wspace if figure is not None
-                  else rcParams["figure.subplot.wspace"])
+                  else mpl.rcParams["figure.subplot.wspace"])
 
         figbox = self._subplot_spec.get_position(figure)
         left, bottom, right, top = figbox.extents
@@ -487,21 +529,21 @@ class GridSpecFromSubplotSpec(GridSpecBase):
 
 class SubplotSpec:
     """
-    Specifies the location of a subplot in a `GridSpec`.
+    The location of a subplot in a `GridSpec`.
 
     .. note::
 
-        Likely, you'll never instantiate a `SubplotSpec` yourself. Instead you
-        will typically obtain one from a `GridSpec` using item-access.
+        Likely, you will never instantiate a `SubplotSpec` yourself. Instead,
+        you will typically obtain one from a `GridSpec` using item-access.
 
     Parameters
     ----------
     gridspec : `~matplotlib.gridspec.GridSpec`
         The GridSpec, which the subplot is referencing.
     num1, num2 : int
-        The subplot will occupy the num1-th cell of the given
-        gridspec.  If num2 is provided, the subplot will span between
-        num1-th cell and num2-th cell *inclusive*.
+        The subplot will occupy the *num1*-th cell of the given
+        *gridspec*.  If *num2* is provided, the subplot will span between
+        *num1*-th cell and *num2*-th cell **inclusive**.
 
         The index starts from 0.
     """
@@ -509,18 +551,56 @@ class SubplotSpec:
         self._gridspec = gridspec
         self.num1 = num1
         self.num2 = num2
-        if gridspec._layoutbox is not None:
-            glb = gridspec._layoutbox
-            # So note that here we don't assign any layout yet,
-            # just make the layoutbox that will contain all items
-            # associated w/ this axis.  This can include other axes like
-            # a colorbar or a legend.
-            self._layoutbox = layoutbox.LayoutBox(
-                    parent=glb,
-                    name=glb.name + '.ss' + layoutbox.seq_id(),
-                    artist=self)
+
+    def __repr__(self):
+        return (f"{self.get_gridspec()}["
+                f"{self.rowspan.start}:{self.rowspan.stop}, "
+                f"{self.colspan.start}:{self.colspan.stop}]")
+
+    @staticmethod
+    def _from_subplot_args(figure, args):
+        """
+        Construct a `.SubplotSpec` from a parent `.Figure` and either
+
+        - a `.SubplotSpec` -- returned as is;
+        - one or three numbers -- a MATLAB-style subplot specifier.
+        """
+        if len(args) == 1:
+            arg, = args
+            if isinstance(arg, SubplotSpec):
+                return arg
+            elif not isinstance(arg, Integral):
+                raise ValueError(
+                    f"Single argument to subplot must be a three-digit "
+                    f"integer, not {arg!r}")
+            try:
+                rows, cols, num = map(int, str(arg))
+            except ValueError:
+                raise ValueError(
+                    f"Single argument to subplot must be a three-digit "
+                    f"integer, not {arg!r}") from None
+        elif len(args) == 3:
+            rows, cols, num = args
         else:
-            self._layoutbox = None
+            raise _api.nargs_error("subplot", takes="1 or 3", given=len(args))
+
+        gs = GridSpec._check_gridspec_exists(figure, rows, cols)
+        if gs is None:
+            gs = GridSpec(rows, cols, figure=figure)
+        if isinstance(num, tuple) and len(num) == 2:
+            if not all(isinstance(n, Integral) for n in num):
+                raise ValueError(
+                    f"Subplot specifier tuple must contain integers, not {num}"
+                )
+            i, j = num
+        else:
+            if not isinstance(num, Integral) or num < 1 or num > rows*cols:
+                raise ValueError(
+                    f"num must be an integer with 1 <= num <= {rows*cols}, "
+                    f"not {num!r}"
+                )
+            i = j = num
+        return gs[i-1:j]
 
     # num2 is a property only to handle the case where it is None and someone
     # mutates num1.
@@ -533,19 +613,6 @@ class SubplotSpec:
     def num2(self, value):
         self._num2 = value
 
-    def __getstate__(self):
-        state = self.__dict__
-        try:
-            state.pop('_layoutbox')
-        except KeyError:
-            pass
-        return state
-
-    def __setstate__(self, state):
-        self.__dict__ = state
-        # layoutboxes don't survive pickling...
-        self._layoutbox = None
-
     def get_gridspec(self):
         return self._gridspec
 
@@ -560,17 +627,6 @@ class SubplotSpec:
         rows, cols = self.get_gridspec().get_geometry()
         return rows, cols, self.num1, self.num2
 
-    def get_rows_columns(self):
-        """
-        Return the subplot row and column numbers as a tuple
-        ``(n_rows, n_cols, row_start, row_stop, col_start, col_stop)``.
-        """
-        gridspec = self.get_gridspec()
-        nrows, ncols = gridspec.get_geometry()
-        row_start, col_start = divmod(self.num1, ncols)
-        row_stop, col_stop = divmod(self.num2, ncols)
-        return nrows, ncols, row_start, row_stop, col_start, col_stop
-
     @property
     def rowspan(self):
         """The rows spanned by this subplot, as a `range` object."""
@@ -581,9 +637,24 @@ class SubplotSpec:
     def colspan(self):
         """The columns spanned by this subplot, as a `range` object."""
         ncols = self.get_gridspec().ncols
-        return range(self.num1 % ncols, self.num2 % ncols + 1)
+        # We explicitly support num2 referring to a column on num1's *left*, so
+        # we must sort the column indices here so that the range makes sense.
+        c1, c2 = sorted([self.num1 % ncols, self.num2 % ncols])
+        return range(c1, c2 + 1)
 
-    def get_position(self, figure, return_all=False):
+    def is_first_row(self):
+        return self.rowspan.start == 0
+
+    def is_last_row(self):
+        return self.rowspan.stop == self.get_gridspec().nrows
+
+    def is_first_col(self):
+        return self.colspan.start == 0
+
+    def is_last_col(self):
+        return self.colspan.stop == self.get_gridspec().ncols
+
+    def get_position(self, figure):
         """
         Update the subplot position from ``figure.subplotpars``.
         """
@@ -597,12 +668,7 @@ class SubplotSpec:
         fig_top = fig_tops[rows].max()
         fig_left = fig_lefts[cols].min()
         fig_right = fig_rights[cols].max()
-        figbox = Bbox.from_extents(fig_left, fig_bottom, fig_right, fig_top)
-
-        if return_all:
-            return figbox, rows[0], cols[0], nrows, ncols
-        else:
-            return figbox
+        return Bbox.from_extents(fig_left, fig_bottom, fig_right, fig_top)
 
     def get_topmost_subplotspec(self):
         """
@@ -641,11 +707,11 @@ class SubplotSpec:
             Number of rows in grid.
 
         ncols : int
-            Number or columns in grid.
+            Number of columns in grid.
 
         Returns
         -------
-        gridspec : `.GridSpecFromSubplotSpec`
+        `.GridSpecFromSubplotSpec`
 
         Other Parameters
         ----------------
