@@ -48,11 +48,11 @@ class Path:
         Draw a line from the current position to the given vertex.
 
     - ``CURVE3`` :  1 control point, 1 endpoint
-        Draw a quadratic Bezier curve from the current position, with the given
+        Draw a quadratic Bézier curve from the current position, with the given
         control point, to the given end point.
 
     - ``CURVE4`` :  2 control points, 1 endpoint
-        Draw a cubic Bezier curve from the current position, with the given
+        Draw a cubic Bézier curve from the current position, with the given
         control points, to the given end point.
 
     - ``CLOSEPOLY`` : 1 vertex (ignored)
@@ -293,15 +293,15 @@ class Path:
     @classmethod
     def make_compound_path_from_polys(cls, XY):
         """
-        Make a compound path object to draw a number
-        of polygons with equal numbers of sides XY is a (numpolys x
-        numsides x 2) numpy array of vertices.  Return object is a
-        :class:`Path`.
+        Make a compound `Path` object to draw a number of polygons with equal
+        numbers of sides.
 
         .. plot:: gallery/misc/histogram_path.py
 
+        Parameters
+        ----------
+        XY : (numpolys, numsides, 2) array
         """
-
         # for each poly: 1 for the MOVETO, (numsides-1) for the LINETO, 1 for
         # the CLOSEPOLY; the vert for the closepoly is ignored but we still
         # need it to keep the codes aligned with the vertices
@@ -316,7 +316,6 @@ class Path:
         codes[numsides::stride] = cls.CLOSEPOLY
         for i in range(numsides):
             verts[i::stride] = XY[:, i]
-
         return cls(verts, codes)
 
     @classmethod
@@ -419,7 +418,7 @@ class Path:
 
     def iter_bezier(self, **kwargs):
         """
-        Iterate over each bezier curve (lines included) in a Path.
+        Iterate over each Bézier curve (lines included) in a Path.
 
         Parameters
         ----------
@@ -429,13 +428,13 @@ class Path:
         Yields
         ------
         B : matplotlib.bezier.BezierSegment
-            The bezier curves that make up the current path. Note in particular
-            that freestanding points are bezier curves of order 0, and lines
-            are bezier curves of order 1 (with two control points).
+            The Bézier curves that make up the current path. Note in particular
+            that freestanding points are Bézier curves of order 0, and lines
+            are Bézier curves of order 1 (with two control points).
         code : Path.code_type
             The code describing what kind of curve is being returned.
             Path.MOVETO, Path.LINETO, Path.CURVE3, Path.CURVE4 correspond to
-            bezier curves with 1, 2, 3, and 4 control points (respectively).
+            Bézier curves with 1, 2, 3, and 4 control points (respectively).
             Path.CLOSEPOLY is a Path.LINETO with the control points correctly
             chosen based on the start/end points of the current stroke.
         """
@@ -461,7 +460,7 @@ class Path:
             elif code == Path.STOP:
                 return
             else:
-                raise ValueError("Invalid Path.code_type: " + str(code))
+                raise ValueError(f"Invalid Path.code_type: {code}")
             prev_vert = verts[-2:]
 
     def cleaned(self, transform=None, remove_nans=False, clip=None,
@@ -513,7 +512,7 @@ class Path:
             by *transform*; i.e. for a correct check, *transform* should
             transform the path into the coordinate system of *point*.
         radius : float, default: 0
-            Add an additional margin on the path in coordinates of *point*.
+            Additional margin on the path in coordinates of *point*.
             The path is extended tangentially by *radius/2*; i.e. if you would
             draw the path with a linewidth of *radius*, all points on the line
             would still be considered to be contained in the area. Conversely,
@@ -563,7 +562,7 @@ class Path:
             by *transform*; i.e. for a correct check, *transform* should
             transform the path into the coordinate system of *points*.
         radius : float, default: 0
-            Add an additional margin on the path in coordinates of *points*.
+            Additional margin on the path in coordinates of *points*.
             The path is extended tangentially by *radius/2*; i.e. if you would
             draw the path with a linewidth of *radius*, all points on the line
             would still be considered to be contained in the area. Conversely,
@@ -688,7 +687,7 @@ class Path:
         polygon/polyline is an Nx2 array of vertices.  In other words,
         each polygon has no ``MOVETO`` instructions or curves.  This
         is useful for displaying in backends that do not support
-        compound paths or Bezier curves.
+        compound paths or Bézier curves.
 
         If *width* and *height* are both non-zero then the lines will
         be simplified so that vertices outside of (0, 0), (width,
@@ -827,7 +826,7 @@ class Path:
 
         Notes
         -----
-        The circle is approximated using 8 cubic Bezier curves, as described in
+        The circle is approximated using 8 cubic Bézier curves, as described in
 
           Lancaster, Don.  `Approximating a Circle or an Ellipse Using Four
           Bezier Cubic Splines <https://www.tinaja.com/glib/ellipse4.pdf>`_.
@@ -938,7 +937,7 @@ class Path:
 
            Masionobe, L.  2003.  `Drawing an elliptical arc using
            polylines, quadratic or cubic Bezier curves
-           <http://www.spaceroots.org/documents/ellipse/index.html>`_.
+           <https://web.archive.org/web/20190318044212/http://www.spaceroots.org/documents/ellipse/index.html>`_.
         """
         halfpi = np.pi * 0.5
 
@@ -1041,7 +1040,6 @@ class Path:
         If *inside* is `True`, clip to the inside of the box, otherwise
         to the outside of the box.
         """
-        # Use make_compound_path_from_polys
         verts = _path.clip_path_to_rect(self, bbox, inside)
         paths = [Path(poly) for poly in verts]
         return self.make_compound_path(*paths)
